%%This is the simulation of the molecular actin and myosin on 1 dimention

%In this simulation we will have myosin and actin molecules. A myosin
%molecule consists of a tail and two heads. It is polarized in the opposite
%orientation of the tail. The polarization of the myosin molecule confines
%it to interacting solely with actin filaments of similar polarization. An
%actin filament consist of multiple actin molecules bound together in a
%chain forming the actin filament. 

%Each head has a rate of attachement to EACH actin filament, however it can
%only be attached to one filament at a time. Once the head is attached it
%will undergo an action, which results in a force on the myosin molecule in
%the direction of its polarization and an equal and opposite force on the
%actin filament. Due to our system being present in the high drag regime,
%the viscous force dominates, resulting in an applied constant force to be
%proportional to a constant velocuty, the relationship between the two
%connected by a drag coefficient. After a certain time, the action is
%completed and the head deataches.

%There is also the presence of diffusion which is modeled by a random walk.

%While the myosin head is attached during the action to the actin filament the structure will
%have a combined drag equal to the sum of the two components.
%clear all;
%close all;


%System Parameters

%% Number of Constituents
%M_num = 10; %Number of Myosin Molecules
A_num = 1; %Number of Actin Filaments

%% Length Characteristics
M_len = 10; %Length of a Myosin molecule (nm)
A_len = 10000; %Average length of an Actin Filament (nm)
A_len_sig = 5; %Standard deviation of the lengths of the Actin Filaments (nm)

Field_length = 20000; %in (nm)

%% Myosin Molecules Position
M_pos = linspace(600, 800, M_num)'; %Position of the Myosin molecule
M_spring_len = zeros(M_num, 2); % lengths of the springs of myosins heads wrt the myosin tail

%% Actin Fiber Position 
%Position is measured from the polarized end
A_pos = 10500*ones(A_num, 2);
A_pos(:, 2) = A_pos(:, 1) - normrnd(A_len, A_len_sig, A_num, 1);

%% Attachement / Detachement rate characteristics
%Tatt = 2; %After Tatt seconds probability of attaching the head is 1/2 (ms)
%Tdatt = 10; %After Tdatt seconds the probability that an attached head will detach is 1/2 (ms)
%Note: using the Rate approach and the Thalf approach is different because the later is density dependant. 
%K_on = 10; %Rate of attachement. How many particles attach after 1 second. This is persumably density independant.
%K_off_max = 20; %Maximum rate of detachment. This corresponds to the rate where the state is maximally relaxed.
%K_off_min = 1; %Minimum rate of detachment. This corresponds to the rate
%where the state is maximally starined.

%NOTE: the times here are in milliseconds (ms)
%T_off = 80; %Average time that the particle will spend being unattached. Is equal to 1/K_on
%T_on_min = 10; %The average time it will spend attached at maximum strain 1/K_off_min
%T_on_max = .5; %The average time it will spend being attaching at minimum strain 1/K_off_max

%% Simulation time characteristics
dt = .005; %time step of the simulation (ms)
Tend = 2000; %run duration (ms)
Tdisplay = 10;

%% Attachement & Force matricies
Attach_Matrix = zeros(M_num, 2); %if 0 then head is not attached. If > 0 then number is the index of the attached actin
Myosin_Force = zeros(M_num, 1); %Force on a myosin molecule from the springs (pN)
Actin_Force = zeros(A_num, 1); %Force on an actin molecule from the attached springs of the myosin heads (pN)
Temp1 = zeros(M_num, 2);
Temp2 = zeros(M_num, 2);

%% Force Characteristics
K = 5; %Spring Constant (in pN/nm)
%Note: 1pN*s/m = 10^-6pN*ms/nm
%Actin_Drag = 9*10^-3; %Actin Drag Coefficient (in pN*ms/nm)
%Myosin_Drag = 6*10^-5; %Myosin Drag Coefficient (in pN*ms/nm)
Actin_Drag = 10; %Actin Drag Coefficient (in pN*ms/nm)
Myosin_Drag = 10000; %Myosin Drag Coefficient (in pN*ms/nm)


%% Step Characteristics
Working_Dis = 5; %Binding Distance on Myosin head to Actin (nm)

%% Time Measurements
Datt_Time_Matrix = zeros(M_num, 2);
Att_Time_Matrix = zeros(M_num, 2);
Duty_Ratio_Matrix = zeros(M_num, 2);
Actin_Pos_Array = [];
Time_Array = [];

T1 = 0;
T2 = 0;
T3 = 0;

%% Simulation
for t = 0:dt:Tend,
    
    %% Move the System
    %[Attach_Matrix, M_spring_len] = Attach_Heads(M_pos, A_pos, Working_Dis, Attach_Matrix, M_spring_len, Tatt/dt, Tdatt/dt);
    
    %[Attach_Matrix, M_spring_len] = Attach_Heads_rate(M_pos, A_pos, Working_Dis, Attach_Matrix, M_spring_len, T_off, T_on_max, T_on_min, dt);
    [Temp1, Temp2] = Attach_Heads_rateC(Attach_Matrix, M_spring_len, Working_Dis, T_off, T_on_max, T_on_min, dt);
    Attach_Matrix = Temp1;
    M_spring_len = Temp2;
    %disp('Attach Heads Function');
   
    %[Myosin_Force, Actin_Force] = Assing_Force(Attach_Matrix, M_spring_len, K, A_num);
    [Myosin_Force, Actin_Force] = Assing_ForceC(Attach_Matrix, M_spring_len, K);
   %disp('Assing Force Function');
   
    %[M_pos, A_pos, M_spring_len] = Move_Molecules(M_pos, A_pos, Myosin_Force, Actin_Force, Myosin_Drag, Actin_Drag, Attach_Matrix, M_spring_len, dt);
    [M_pos, A_pos, M_spring_len] = Move_MoleculesC(M_pos, A_pos, Myosin_Force, Actin_Force, Myosin_Drag, Actin_Drag, Attach_Matrix, M_spring_len, dt);
    %disp('Move Molecules Function');
    

    %% Measure Parameters
    %Measuring Average time for attachment
        Datt_Time_Matrix = Datt_Time_Matrix + dt*(Attach_Matrix == 0);
        
     %Measuring average time of dettachment
        Att_Time_Matrix = Att_Time_Matrix + dt*(Attach_Matrix > 0);
        
        
        if ((mod(t, .1) == 0) && (t > 20)),
            Actin_Pos_Array = [Actin_Pos_Array, A_pos(:, 1)];
            Time_Array = [Time_Array, t];
        end
    %disp('Measure Parameters');
    
    
    
    
   %% Display
    %if mod(t, Tdisplay) == 0,
        %pause(.01);
        %Display_Molecular(M_pos, A_pos, M_len, M_spring_len, Working_Dis, Attach_Matrix), axis([0 Field_length -10 10]), title(t);
    %end
    
end

Duty_Ratio_Matrix = Att_Time_Matrix./(Att_Time_Matrix + Datt_Time_Matrix);
duty_ratio = mean(mean(Duty_Ratio_Matrix));
Avg_Velocity = mean(diff(Actin_Pos_Array))*10;
Disp_Title = ['Actin Position vs Time at Mnum = ', num2str(M_num)];
figure, plot(Time_Array, Actin_Pos_Array), title(Disp_Title);
beep;

